@@ -3,5 +3,5 @@
 """
 An exception handler for interactive debugging
 """
-from middleware import EvalException
+from paste.evalexception.middleware import EvalException
 
