@@ -5,8 +5,9 @@ Middleware for logging requests, using Apache combined log format
 """
 
 import logging
+import six
 import time
-import urllib
+from six.moves.urllib.parse import quote
 
 class TransLogger(object):
     """
@@ -50,10 +51,11 @@ class TransLogger(object):
 
     def __call__(self, environ, start_response):
         start = time.localtime()
-        req_uri = urllib.quote(environ.get('SCRIPT_NAME', '')
+        req_uri = quote(environ.get('SCRIPT_NAME', '')
                                + environ.get('PATH_INFO', ''))
         if environ.get('QUERY_STRING'):
             req_uri += '?'+environ['QUERY_STRING']
+        method = environ['REQUEST_METHOD']
         def replacement_start_response(status, headers, exc_info=None):
             # @@: Ideally we would count the bytes going by if no
             # content-length header was provided; but that does add
@@ -62,11 +64,11 @@ class TransLogger(object):
             for name, value in headers:
                 if name.lower() == 'content-length':
                     bytes = value
-            self.write_log(environ, req_uri, start, status, bytes)
+            self.write_log(environ, method, req_uri, start, status, bytes)
             return start_response(status, headers)
         return self.application(environ, replacement_start_response)
 
-    def write_log(self, environ, req_uri, start, status, bytes):
+    def write_log(self, environ, method, req_uri, start, status, bytes):
         if bytes is None:
             bytes = '-'
         if time.daylight:
@@ -77,10 +79,15 @@ class TransLogger(object):
                 offset = "+%0.4d" % (offset)
         elif offset < 0:
                 offset = "%0.4d" % (offset)
+        remote_addr = '-'
+        if environ.get('HTTP_X_FORWARDED_FOR'):
+            remote_addr = environ['HTTP_X_FORWARDED_FOR']
+        elif environ.get('REMOTE_ADDR'):
+            remote_addr = environ['REMOTE_ADDR']
         d = {
-            'REMOTE_ADDR': environ.get('REMOTE_ADDR') or '-',
+            'REMOTE_ADDR': remote_addr,
             'REMOTE_USER': environ.get('REMOTE_USER') or '-',
-            'REQUEST_METHOD': environ['REQUEST_METHOD'],
+            'REQUEST_METHOD': method,
             'REQUEST_URI': req_uri,
             'HTTP_VERSION': environ.get('SERVER_PROTOCOL'),
             'time': time.strftime('%d/%b/%Y:%H:%M:%S ', start) + offset,
@@ -100,9 +107,9 @@ def make_filter(
     setup_console_handler=True,
     set_logger_level=logging.DEBUG):
     from paste.util.converters import asbool
-    if isinstance(logging_level, basestring):
+    if isinstance(logging_level, (six.binary_type, six.text_type)):
         logging_level = logging._levelNames[logging_level]
-    if isinstance(set_logger_level, basestring):
+    if isinstance(set_logger_level, (six.binary_type, six.text_type)):
         set_logger_level = logging._levelNames[set_logger_level]
     return TransLogger(
         app,
