@@ -5,6 +5,7 @@ WSGI applications that parse the URL and dispatch to on-disk resources
 """
 
 import os
+import six
 import sys
 import imp
 import mimetypes
@@ -16,7 +17,7 @@ from paste import request
 from paste import fileapp
 from paste.util import import_string
 from paste import httpexceptions
-from httpheaders import ETAG
+from .httpheaders import ETAG
 from paste.util import converters
 
 class NoDefault(object):
@@ -215,9 +216,7 @@ class URLParser(object):
         exc = httpexceptions.HTTPNotFound(
             'The resource at %s could not be found'
             % request.construct_url(environ),
-            comment='SCRIPT_NAME=%r; PATH_INFO=%r; looking in %r; debug: %s'
-            % (environ.get('SCRIPT_NAME'), environ.get('PATH_INFO'),
-               self.directory, debug_message or '(none)'))
+            comment=debug_message)
         return exc.wsgi_application(environ, start_response)
 
     def add_slash(self, environ, start_response):
@@ -300,7 +299,7 @@ class URLParser(object):
         ``environ`` and ``filename``, and returns a WSGI application.
         """
         d = cls.global_constructors
-        assert not d.has_key(extension), (
+        assert extension not in d, (
             "A constructor already exists for the extension %r (%r) "
             "when attemption to register constructor %r"
             % (extension, d[extension], constructor))
@@ -364,13 +363,13 @@ def load_module(environ, filename):
                                  environ['wsgi.errors'])
 
 def load_module_from_name(environ, filename, module_name, errors):
-    if sys.modules.has_key(module_name):
+    if module_name in sys.modules:
         return sys.modules[module_name]
     init_filename = os.path.join(os.path.dirname(filename), '__init__.py')
     if not os.path.exists(init_filename):
         try:
             f = open(init_filename, 'w')
-        except (OSError, IOError), e:
+        except (OSError, IOError) as e:
             errors.write(
                 'Cannot write __init__.py file into directory %s (%s)\n'
                 % (os.path.dirname(filename), e))
@@ -378,7 +377,7 @@ def load_module_from_name(environ, filename, module_name, errors):
         f.write('#\n')
         f.close()
     fp = None
-    if sys.modules.has_key(module_name):
+    if module_name in sys.modules:
         return sys.modules[module_name]
     if '.' in module_name:
         parent_name = '.'.join(module_name.split('.')[:-1])
@@ -419,7 +418,6 @@ def make_py(parser, environ, filename):
 URLParser.register_constructor('.py', make_py)
 
 class StaticURLParser(object):
-    
     """
     Like ``URLParser`` but only serves static files.
 
@@ -430,20 +428,13 @@ class StaticURLParser(object):
 
     def __init__(self, directory, root_directory=None,
                  cache_max_age=None):
-        if os.path.sep != '/':
-            directory = directory.replace(os.path.sep, '/')
-        self.directory = os.path.normcase(os.path.abspath(directory))
-        self.root_directory = root_directory
-        if root_directory is not None:
-            self.root_directory = os.path.normpath(self.root_directory)
-        else:
-            self.root_directory = directory
-        self.root_directory = os.path.normcase(os.path.normpath(
-            os.path.abspath(self.root_directory)))
+        self.directory = self.normpath(directory)
+        self.root_directory = self.normpath(root_directory or directory)
         self.cache_max_age = cache_max_age
-        if os.path.sep != '/':
-            directory = directory.replace('/', os.path.sep)
-            self.root_directory = self.root_directory.replace('/', os.path.sep)
+
+    def normpath(path):
+        return os.path.normcase(os.path.abspath(path))
+    normpath = staticmethod(normpath)
 
     def __call__(self, environ, start_response):
         path_info = environ.get('PATH_INFO', '')
@@ -454,20 +445,15 @@ class StaticURLParser(object):
             filename = 'index.html'
         else:
             filename = request.path_info_pop(environ)
-        full = os.path.normcase(os.path.normpath(
-            os.path.join(self.directory, filename)))
-        if os.path.sep != '/':
-            full = full.replace('/', os.path.sep)
-        if self.root_directory is not None and not full.startswith(self.root_directory):
+        full = self.normpath(os.path.join(self.directory, filename))
+        if not full.startswith(self.root_directory):
             # Out of bounds
             return self.not_found(environ, start_response)
         if not os.path.exists(full):
             return self.not_found(environ, start_response)
         if os.path.isdir(full):
             # @@: Cache?
-            child_root = self.root_directory is not None and \
-                self.root_directory or self.directory
-            return self.__class__(full, root_directory=child_root, 
+            return self.__class__(full, root_directory=self.root_directory,
                                   cache_max_age=self.cache_max_age)(environ,
                                                                    start_response)
         if environ.get('PATH_INFO') and environ.get('PATH_INFO') != '/':
@@ -477,10 +463,12 @@ class StaticURLParser(object):
             mytime = os.stat(full).st_mtime
             if str(mytime) == if_none_match:
                 headers = []
+                ## FIXME: probably should be
+                ## ETAG.update(headers, '"%s"' % mytime)
                 ETAG.update(headers, mytime)
                 start_response('304 Not Modified', headers)
                 return [''] # empty body
-        
+
         fa = self.make_app(full)
         if self.cache_max_age:
             fa.cache_control(max_age=self.cache_max_age)
@@ -503,7 +491,7 @@ class StaticURLParser(object):
             'automatically.' % url,
             headers=[('location', url)])
         return exc.wsgi_application(environ, start_response)
-        
+
     def not_found(self, environ, start_response, debug_message=None):
         exc = httpexceptions.HTTPNotFound(
             'The resource at %s could not be found'
@@ -517,7 +505,7 @@ class StaticURLParser(object):
         exc = httpexceptions.HTTPNotFound(
             'The trailing path %r is not allowed' % environ['PATH_INFO'])
         return exc.wsgi_application(environ, start_response)
-    
+
     def __repr__(self):
         return '<%s %r>' % (self.__class__.__name__, self.directory)
 
@@ -525,7 +513,7 @@ def make_static(global_conf, document_root, cache_max_age=None):
     """
     Return a WSGI application that serves a directory (configured
     with document_root)
-    
+
     cache_max_age - integer specifies CACHE_CONTROL max_age in seconds
     """
     if cache_max_age is not None:
@@ -538,7 +526,7 @@ class PkgResourcesParser(StaticURLParser):
     def __init__(self, egg_or_spec, resource_name, manager=None, root_resource=None):
         if pkg_resources is None:
             raise NotImplementedError("This class requires pkg_resources.")
-        if isinstance(egg_or_spec, (str, unicode)):
+        if isinstance(egg_or_spec, (six.binary_type, six.text_type)):
             self.egg = pkg_resources.get_distribution(egg_or_spec)
         else:
             self.egg = egg_or_spec
@@ -580,21 +568,21 @@ class PkgResourcesParser(StaticURLParser):
                                   root_resource=child_root)(environ, start_response)
         if environ.get('PATH_INFO') and environ.get('PATH_INFO') != '/':
             return self.error_extra_path(environ, start_response)
-        
+
         type, encoding = mimetypes.guess_type(resource)
         if not type:
             type = 'application/octet-stream'
         # @@: I don't know what to do with the encoding.
         try:
             file = self.egg.get_resource_stream(self.manager, resource)
-        except (IOError, OSError), e:
+        except (IOError, OSError) as e:
             exc = httpexceptions.HTTPForbidden(
                 'You are not permitted to view this file (%s)' % e)
             return exc.wsgi_application(environ, start_response)
         start_response('200 OK',
                        [('content-type', type)])
         return fileapp._FileIter(file)
-        
+
     def not_found(self, environ, start_response, debug_message=None):
         exc = httpexceptions.HTTPNotFound(
             'The resource at %s could not be found'
@@ -637,16 +625,15 @@ def make_url_parser(global_conf, directory, base_python_name,
         hide_extensions = global_conf.get(
             'hide_extensions', ('.pyc', 'bak', 'py~'))
     hide_extensions = converters.aslist(hide_extensions)
-    
+
     if ignore_extensions is None:
         ignore_extensions = global_conf.get(
             'ignore_extensions', ())
     ignore_extensions = converters.aslist(ignore_extensions)
     # There's no real way to set constructors currently...
-    
+
     return URLParser({}, directory, base_python_name,
                      index_names=index_names,
                      hide_extensions=hide_extensions,
                      ignore_extensions=ignore_extensions,
                      **constructor_conf)
-
