@@ -55,7 +55,6 @@ correctly.
 """
 from datetime import timedelta, time, date
 from time import localtime
-import string
 
 __all__ = ['parse_timedelta', 'normalize_timedelta',
            'parse_time', 'normalize_time',
@@ -63,7 +62,7 @@ __all__ = ['parse_timedelta', 'normalize_timedelta',
 
 def _number(val):
     try:
-        return string.atoi(val)
+        return int(val)
     except:
         return None
 
@@ -76,17 +75,16 @@ def parse_timedelta(val):
     """
     if not val:
         return None
-    val = string.lower(val)
+    val = val.lower()
     if "." in val:
         val = float(val)
         return timedelta(hours=int(val), minutes=60*(val % 1.0))
     fHour = ("h" in val or ":" in val)
     fMin  = ("m" in val or ":" in val)
-    fFraction = "." in val
     for noise in "minu:teshour()":
-        val = string.replace(val, noise, ' ')
-    val = string.strip(val)
-    val = string.split(val)
+        val = val.replace(noise, ' ')
+    val = val.strip()
+    val = val.split()
     hr = 0.0
     mi = 0
     val.reverse()
@@ -121,12 +119,12 @@ def parse_time(val):
     if not val:
         return None
     hr = mi = 0
-    val = string.lower(val)
-    amflag = (-1 != string.find(val, 'a'))  # set if AM is found
-    pmflag = (-1 != string.find(val, 'p'))  # set if PM is found
+    val = val.lower()
+    amflag = (-1 != val.find('a'))  # set if AM is found
+    pmflag = (-1 != val.find('p'))  # set if PM is found
     for noise in ":amp.":
-        val = string.replace(val, noise, ' ')
-    val = string.split(val)
+        val = val.replace(noise, ' ')
+    val = val.split()
     if len(val) > 1:
         hr = int(val[0])
         mi = int(val[1])
@@ -198,7 +196,7 @@ _wkdy = ("mon", "tue", "wed", "thu", "fri", "sat", "sun")
 def parse_date(val):
     if not(val):
         return None
-    val = string.lower(val)
+    val = val.lower()
     now = None
 
     # optimized check for YYYY-MM-DD
@@ -243,9 +241,9 @@ def parse_date(val):
     # ok, standard parsing
     yr = mo = dy = None
     for noise in ('/', '-', ',', '*'):
-        val = string.replace(val, noise, ' ')
+        val = val.replace(noise, ' ')
     for noise in _wkdy:
-        val = string.replace(val, noise, ' ')
+        val = val.replace(noise, ' ')
     out = []
     last = False
     ldig = False
@@ -260,7 +258,7 @@ def parse_date(val):
                 ldig = False
             last = True
         out.append(ch)
-    val = string.split("".join(out))
+    val = "".join(out).split()
     if 3 == len(val):
         a = _number(val[0])
         b = _number(val[1])
@@ -273,7 +271,7 @@ def parse_date(val):
             else:  # 1999 Jun 23
                 mo = _month(val[1])
                 dy = c
-        elif a > 0:
+        elif a is not None and a > 0:
             yr = c
             if len(val[2]) < 4:
                 raise TypeError("four digit year required")
@@ -292,19 +290,19 @@ def parse_date(val):
     elif 2 == len(val):
         a = _number(val[0])
         b = _number(val[1])
-        if a > 999:
+        if a is not None and a > 999:
             yr = a
             dy = 1
-            if b > 0: # 1999 6
+            if b is not None and b > 0: # 1999 6
                 mo = b
             else: # 1999 Jun
                 mo = _month(val[1])
-        elif a > 0:
-            if b > 999: # 6 1999
+        elif a is not None and a > 0:
+            if b is not None and b > 999: # 6 1999
                 mo = a
                 yr = b
                 dy = 1
-            elif b > 0: # 6 23
+            elif b is not None and b > 0: # 6 23
                 mo = a
                 dy = b
             else: # 23 Jun
@@ -331,7 +329,7 @@ def parse_date(val):
                 mo = _number(val[4:6])
                 dy = _number(val[6:])
             elif len(val) in (3,4):
-                if v > 1300: # 2004
+                if v is not None and v > 1300: # 2004
                     yr = v
                     mo = 1
                     dy = 1
