@@ -5,6 +5,8 @@
 A module of many disparate routines.
 """
 
+from __future__ import print_function
+
 # functions which moved to paste.request and paste.response
 # Deprecated around 15 Dec 2005
 from paste.request import get_cookies, parse_querystring, parse_formvars
@@ -13,10 +15,10 @@ from paste.response import HeaderDict, has_header, header_value, remove_header
 from paste.response import error_body_response, error_response, error_response_app
 
 from traceback import print_exception
-import urllib
-from cStringIO import StringIO
+import six
 import sys
-from urlparse import urlsplit
+from six.moves import cStringIO as StringIO
+from six.moves.urllib.parse import unquote, urlsplit
 import warnings
 
 __all__ = ['add_close', 'add_start_close', 'capture_output', 'catch_errors',
@@ -42,7 +44,11 @@ class add_close(object):
         return self
 
     def next(self):
-        return self.app_iter.next()
+        return self.__next__()
+
+    # Python 3 uses __next__ instead of next
+    def __next__(self):
+        return next(self.app_iter)
 
     def close(self):
         self._closed = True
@@ -53,10 +59,9 @@ class add_close(object):
     def __del__(self):
         if not self._closed:
             # We can't raise an error or anything at this stage
-            print >> sys.stderr, (
-                "Error: app_iter.close() was not called when finishing "
+            print("Error: app_iter.close() was not called when finishing "
                 "WSGI request. finalization function %s not called"
-                % self.close_func)
+                  % self.close_func, file=sys.stderr)
 
 class add_start_close(object):
     """
@@ -80,7 +85,8 @@ class add_start_close(object):
         if self.first:
             self.start_func()
             self.first = False
-        return self.app_iter.next()
+        return next(self.app_iter)
+    __next__ = next
 
     def close(self):
         self._closed = True
@@ -92,10 +98,9 @@ class add_start_close(object):
     def __del__(self):
         if not self._closed:
             # We can't raise an error or anything at this stage
-            print >> sys.stderr, (
-                "Error: app_iter.close() was not called when finishing "
+            print("Error: app_iter.close() was not called when finishing "
                 "WSGI request. finalization function %s not called"
-                % self.close_func)
+                  % self.close_func, file=sys.stderr)
 
 class chained_app_iters(object):
 
@@ -132,15 +137,14 @@ class chained_app_iters(object):
             except:
                 got_exc = sys.exc_info()
         if got_exc:
-            raise got_exc[0], got_exc[1], got_exc[2]
+            six.reraise(got_exc[0], got_exc[1], got_exc[2])
 
     def __del__(self):
         if not self._closed:
             # We can't raise an error or anything at this stage
-            print >> sys.stderr, (
-                "Error: app_iter.close() was not called when finishing "
+            print("Error: app_iter.close() was not called when finishing "
                 "WSGI request. finalization function %s not called"
-                % self.close_func)
+                  % self.close_func, file=sys.stderr)
 
 class encode_unicode_app_iter(object):
     """
@@ -158,10 +162,11 @@ class encode_unicode_app_iter(object):
         return self
 
     def next(self):
-        content = self.app_iter.next()
-        if isinstance(content, unicode):
+        content = next(self.app_iter)
+        if isinstance(content, six.text_type):
             content = content.encode(self.encoding, self.errors)
         return content
+    __next__ = next
 
     def close(self):
         if hasattr(self.app_iterable, 'close'):
@@ -283,19 +288,19 @@ def raw_interactive(application, path='', raise_on_wsgi_error=False,
     if raise_on_wsgi_error:
         errors = ErrorRaiser()
     else:
-        errors = StringIO()
+        errors = six.BytesIO()
     basic_environ = {
         # mandatory CGI variables
         'REQUEST_METHOD': 'GET',     # always mandatory
         'SCRIPT_NAME': '',           # may be empty if app is at the root
         'PATH_INFO': '',             # may be empty if at root of app
         'SERVER_NAME': 'localhost',  # always mandatory
-        'SERVER_PORT': '80',         # always mandatory 
+        'SERVER_PORT': '80',         # always mandatory
         'SERVER_PROTOCOL': 'HTTP/1.0',
         # mandatory wsgi variables
         'wsgi.version': (1, 0),
         'wsgi.url_scheme': 'http',
-        'wsgi.input': StringIO(''),
+        'wsgi.input': six.BytesIO(),
         'wsgi.errors': errors,
         'wsgi.multithread': False,
         'wsgi.multiprocess': False,
@@ -303,7 +308,9 @@ def raw_interactive(application, path='', raise_on_wsgi_error=False,
         }
     if path:
         (_, _, path_info, query, fragment) = urlsplit(str(path))
-        path_info = urllib.unquote(path_info)
+        path_info = unquote(path_info)
+        # urlsplit returns unicode so coerce it back to str
+        path_info, query = str(path_info), str(query)
         basic_environ['PATH_INFO'] = path_info
         if query:
             basic_environ['QUERY_STRING'] = query
@@ -314,8 +321,8 @@ def raw_interactive(application, path='', raise_on_wsgi_error=False,
         and 'HTTP_HOST' not in basic_environ):
         basic_environ['HTTP_HOST'] = basic_environ['SERVER_NAME']
     istream = basic_environ['wsgi.input']
-    if isinstance(istream, str):
-        basic_environ['wsgi.input'] = StringIO(istream)
+    if isinstance(istream, bytes):
+        basic_environ['wsgi.input'] = six.BytesIO(istream)
         basic_environ['CONTENT_LENGTH'] = len(istream)
     data = {}
     output = []
@@ -326,7 +333,7 @@ def raw_interactive(application, path='', raise_on_wsgi_error=False,
             try:
                 if headers_sent:
                     # Re-raise original exception only if headers sent
-                    raise exc_info[0], exc_info[1], exc_info[2]
+                    six.reraise(exc_info[0], exc_info[1], exc_info[2])
             finally:
                 # avoid dangling circular reference
                 exc_info = None
@@ -342,15 +349,15 @@ def raw_interactive(application, path='', raise_on_wsgi_error=False,
     try:
         try:
             for s in app_iter:
-                if not isinstance(s, str):
+                if not isinstance(s, six.binary_type):
                     raise ValueError(
-                        "The app_iter response can only contain str (not "
+                        "The app_iter response can only contain bytes (not "
                         "unicode); got: %r" % s)
                 headers_sent.append(True)
                 if not headers_set:
                     raise AssertionError("Content sent w/o headers!")
                 output.append(s)
-        except TypeError, e:
+        except TypeError as e:
             # Typically "iteration over non-sequence", so we want
             # to give better debugging information...
             e.args = ((e.args[0] + ' iterable: %r' % app_iter),) + e.args[1:]
@@ -358,7 +365,7 @@ def raw_interactive(application, path='', raise_on_wsgi_error=False,
     finally:
         if hasattr(app_iter, 'close'):
             app_iter.close()
-    return (data['status'], data['headers'], ''.join(output),
+    return (data['status'], data['headers'], b''.join(output),
             errors.getvalue())
 
 class ErrorRaiser(object):
@@ -404,7 +411,7 @@ def dump_environ(environ, start_response):
     variables out as a plain text response.
     """
     output = []
-    keys = environ.keys()
+    keys = list(environ.keys())
     keys.sort()
     for k in keys:
         v = str(environ[k]).replace("\n","\n    ")
@@ -415,6 +422,8 @@ def dump_environ(environ, start_response):
         output.append(environ['wsgi.input'].read(int(content_length)))
         output.append("\n")
     output = "".join(output)
+    if six.PY3:
+        output = output.encode('utf8')
     headers = [('Content-Type', 'text/plain'),
                ('Content-Length', str(len(output)))]
     start_response("200 OK", headers)
@@ -434,8 +443,8 @@ def capture_output(environ, start_response, application):
 
     Sends status and header, but *not* body.  Returns (status,
     headers, body).  Typically this is used like:
-    
-    .. code-block:: Python
+
+    .. code-block:: python
 
         def dehtmlifying_middleware(application):
             def replacement_app(environ, start_response):
@@ -485,13 +494,14 @@ def intercept_output(environ, application, conditional=None,
     ``capture_output``)
 
     Typically this is used like:
-    
-    .. code-block:: Python
+
+    .. code-block:: python
 
         def dehtmlifying_middleware(application):
             def replacement_app(environ, start_response):
                 status, headers, body = intercept_output(
                     environ, application)
+                start_response(status, headers)
                 content_type = header_value(headers, 'content-type')
                 if (not content_type
                     or not content_type.startswith('text/html')):
@@ -506,8 +516,8 @@ def intercept_output(environ, application, conditional=None,
     ``start_response`` will be called and ``(None, None, app_iter)``
     will be returned.  You must detect that in your code and return
     the app_iter, like:
-    
-    .. code-block:: Python
+
+    .. code-block:: python
 
         def dehtmlifying_middleware(application):
             def replacement_app(environ, start_response):
@@ -517,6 +527,7 @@ def intercept_output(environ, application, conditional=None,
                     start_response)
                 if status is None:
                     return body
+                start_response(status, headers)
                 body = re.sub(r'<.*?>', '', body)
                 return [body]
             return replacement_app
@@ -590,4 +601,4 @@ for _name in __all__:
 if __name__ == '__main__':
     import doctest
     doctest.testmod()
-    
+
